package gov.usgs.earthquake.nshmp.gmm;

/**
 * Models of additional epistemic uncertainty in ground motion. Available models
 * are currently parameterized in magnitude and distance.
 * 
 * @author U.S. Geological Survey
 */
public enum UncertaintyModel {

  /** Model derived from the PEER NGA-West1 project. */
  NGA_WEST1(new double[] { 0.375, 0.23, 0.4, 0.21, 0.225, 0.36, 0.245, 0.23, 0.31 }),

  /** Model derived from the PEER NGA-West2 project. */
  NGA_WEST2(new double[] { 0.37, 0.25, 0.4, 0.22, 0.23, 0.36, 0.22, 0.23, 0.33 });

  private final double[][] values;

  private UncertaintyModel(double[] values) {
    this.values = initValues(values);
  }

  /**
   * Return the epistemic uncertainty for the supplied magnitude and distance in
   * natural log units of ground motion.
   */
  public double value(double m, double r) {
    int mi = (m < 6) ? 0 : (m < 7) ? 1 : 2;
    int ri = (r < 10) ? 0 : (r < 30) ? 1 : 2;
    return values[ri][mi];

  }

  private static double[][] initValues(double[] v) {
    return new double[][] {
        { v[0], v[1], v[2] },
        { v[3], v[4], v[5] },
        { v[6], v[7], v[8] }
    };
  }
}
