package gov.usgs.earthquake.nshmp.mfd;

import java.util.Iterator;

import gov.usgs.earthquake.nshmp.data.XyPoint;
import gov.usgs.earthquake.nshmp.data.XySequence;

/**
 * An incremental magnitude frequency distribution (MFD) that defines the rates
 * of earthquakes at one or more magnitudes.
 * 
 * <p>MFDs may be initialized using the factory methods supplied in
 * {@link Mfds}. Each factory method returns an {@link MfdBuilder}, which may be
 * used to modify and shape the MFD prior to calling {@link MfdBuilder#build()}.
 * Internally, an {@code Mfd} is backed by an {@link XySequence}.
 * 
 * @author Peter Powers
 * @see Mfds
 * @see XySequence
 */
public class Mfd implements Iterable<XyPoint> {

  private final XySequence mfd;

  Mfd(XySequence mfd) {
    this.mfd = mfd;
  }

  @Override
  public Iterator<XyPoint> iterator() {
    return mfd.iterator();
  }

  /** Magnitude-frequency distribution (MFD) type identifier. */
  public enum Type {

    /** An MFD with a single magnitude and rate. */
    SINGLE,

    /** An incremental Gutenberg-Richter MFD. */
    GR,

    /** A Gutenberg-Richter MFD with a tapered upper tail. */
    GR_TAPER,

    /** An MFD defining multiple magnitudes with varying rates. */
    INCR;
  }

}
