package gov.usgs.earthquake.nshmp;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;

@SuppressWarnings("javadoc")
public final class EarthquakesTests {

  @Test
  public final void testCheckDepths() {
    assertEquals(5.0, Earthquakes.checkDepth(5.0), 0.0);
    assertEquals(5.0, Earthquakes.checkCrustalDepth(5.0), 0.0);
    assertEquals(15.0, Earthquakes.checkCrustalWidth(15.0), 0.0);
    assertEquals(10.0, Earthquakes.checkInterfaceDepth(10.0), 0.0);
    assertEquals(40.0, Earthquakes.checkInterfaceWidth(40.0), 0.0);
    assertEquals(30.0, Earthquakes.checkSlabDepth(30.0), 0.0);
  }

  @Test
  public final void checkExpcetionTests() {
    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkDepth(-5.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkDepth(700.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkCrustalDepth(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkCrustalDepth(40.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkCrustalWidth(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkCrustalWidth(60.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkInterfaceDepth(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkInterfaceDepth(60.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkInterfaceWidth(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkInterfaceWidth(200.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkSlabDepth(19.9);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkSlabDepth(700.1);
    });
  }

  @Test
  public final void testCheckMagnitude() {
    assertEquals(6.5, Earthquakes.checkMagnitude(6.5), 0.0);

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkMagnitude(-2.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Earthquakes.checkMagnitude(9.8);
    });
  }

  @Test
  public final void testMagToMoment() {
    double moExpect = 6.3095734e18;
    double magInput = 6.5;
    double tolerance = 1e11; // 8 sig figs
    assertEquals(moExpect, Earthquakes.magToMoment(magInput), tolerance);
  }

  @Test
  public final void testMomentToMag() {
    double magExpect = 6.5;
    double moInput = 6.3095734e18;
    double tolerance = 1e-8; // 8 sig figs
    assertEquals(magExpect, Earthquakes.momentToMag(moInput), tolerance);
  }

  @Test
  public final void testMoment() {
    double fLength = 20 * 1000.0; // 20 km
    double fWidth = 10 * 1000.0; // 10 km
    double slip = 0.02; // 20 mm
    double moExpect = 1.2e17;
    assertEquals(moExpect, Earthquakes.moment(fLength * fWidth, slip), 0.0);
  }

  @Test
  public final void testSlip() {
    double fLength = 20 * 1000.0; // 20 km
    double fWidth = 10 * 1000.0; // 10 km
    double moment = 1.2e17;
    double slipExpect = 0.02; // 20 mm
    assertEquals(slipExpect, Earthquakes.slip(fLength * fWidth, moment), 0.0);
  }

}
