package gov.usgs.earthquake.nshmp.eq.model;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.eq.fault.surface.RuptureFloating;

class SourceConfigTests {

  static final Path RESOURCES = Paths.get("src/test/resources/eq/model/source-configs");

  static final Path CONFIG_1 = RESOURCES.resolve("source-config-1.json");
  static final Path CONFIG_2 = RESOURCES.resolve("source-config-2.json");
  static final Path CONFIG_3 = RESOURCES.resolve("source-config-3.json");

  @Test
  void testCreate() throws IOException {
    SourceConfig config = SourceConfig.fromFile(CONFIG_1).build();
    testEqualsConfig1(config);
  }

  @Test
  void testCopyOf() throws IOException {
    SourceConfig config = SourceConfig.fromFile(CONFIG_1).build();
    SourceConfig copy = SourceConfig.copyOf(config).build();
    System.out.println(copy);
    testEqualsConfig1(copy);
  }

  // TODO clean
  // @Test
  // void testExtendAll() throws IOException {
  // SourceConfig config = SourceConfig.fromFile(CONFIG_1)
  // .extend(SourceConfig.fromFile(CONFIG_2))
  // .build();
  // assertEquals(2.0, config.surfaceSpacing, 0.0);
  // assertEquals(RuptureFloating.OFF, config.ruptureFloating);
  // assertEquals(true, config.ruptureVariability);
  // assertEquals(PointSourceType.POINT, config.pointSourceType);
  // assertEquals(0.5, config.areaGridSpacing, 0.0);
  // }
  //
  // @Test
  // void testExtendNone() throws IOException {
  // SourceConfig config = SourceConfig.fromFile(CONFIG_1)
  // .extend(SourceConfig.fromFile(CONFIG_3))
  // .build();
  // testEqualsConfig1(config);
  // }

  private void testEqualsConfig1(SourceConfig config) {
    assertEquals(1.0, config.surfaceSpacing, 0.0);
    assertEquals(RuptureFloating.NSHM, config.ruptureFloating);
    assertEquals(false, config.ruptureVariability);
    assertEquals(PointSourceType.FINITE, config.pointSourceType);
    assertEquals(0.1, config.areaGridSpacing, 0.0);
  }

  static final String TO_STRING =
      "source-config: " + CONFIG_1.toAbsolutePath().normalize() + "\n" +
          "  .surface-spacing:     1.0\n" +
          "  .rupture-floating:    NSHM\n" +
          "  .rupture-variability: false\n" +
          "  .point-source-type:   FINITE\n" +
          "  .area-grid-spacing:   0.1";

  @Test
  void testToString() throws IOException {
    SourceConfig config = SourceConfig.fromFile(CONFIG_1).build();
    assertEquals(TO_STRING, config.toString());
  }

  @Test
  public void testRepeatBuild() throws Exception {
    assertThrows(IllegalStateException.class, () -> {
      SourceConfig.Builder b = SourceConfig.fromFile(CONFIG_1);
      b.build();
      b.build();
    });
  }

}
